@@ -1,4 +1,5 @@
-/* Copyright (C) 2012, 2014, 2016, 2019  Olga Yakovleva <yakovleva.o.v@gmail.com> */
+/* Copyright (C) 2012, 2014, 2016, 2019, 2021  Olga Yakovleva <olga@rhvoice.org> */
+/* Copyright (C) 2022 Non-Routine LLC.  <lp@louderpages.org> */
 
 /* This program is free software: you can redistribute it and/or modify */
 /* it under the terms of the GNU Lesser General Public License as published by */
@@ -11,7 +12,7 @@
 /* GNU Lesser General Public License for more details. */
 
 /* You should have received a copy of the GNU Lesser General Public License */
-/* along with this program.  If not, see <http://www.gnu.org/licenses/>. */
+/* along with this program.  If not, see <https://www.gnu.org/licenses/>. */
 
 #include <algorithm>
 #include <functional>
@@ -29,9 +30,11 @@
 #include "core/kyrgyz.hpp"
 #include "core/tatar.hpp"
 #include "core/brazilian_portuguese.hpp"
+#include "core/macedonian.hpp"
 #include "core/stress_pattern.hpp"
 #include "core/event_logger.hpp"
 #include "core/emoji.hpp"
+#include "core/data_only_language.hpp"
 #include <iostream>
 
 namespace RHVoice
@@ -390,6 +393,26 @@ namespace RHVoice
       }
     };
 
+    struct feat_word_break: public feature_function
+    {
+      feat_word_break():
+        feature_function("word_break")
+      {
+      }
+
+      value eval(const item& word) const
+      {
+        unsigned int result=4;
+        const item& word_in_utt=word.as("Word");
+        if(word_in_utt.has_next())
+          {
+            const item& word_in_phrase=word.as("Phrase");
+            result=word_in_phrase.has_next()?1:3;
+              }
+        return result;
+      }
+    };
+
     struct feat_phrases_in: public feature_function
     {
       feat_phrases_in():
@@ -556,6 +579,12 @@ else
     spell_fst(path::join(info_.get_data_path(),"spell.fst")),
     downcase_fst(path::join(info_.get_data_path(),"downcase.fst"))
   {
+    config cfg;
+    cfg.register_setting(lcfg.tok_eos);
+cfg.register_setting(lcfg.tok_sent);
+    cfg.register_setting(lcfg.ph_flags);
+    cfg.register_setting(lcfg.g2p_case);
+    cfg.load(path::join(info_.get_data_path(),"language.conf"));
     try
       {
         english_phone_mapping_fst.reset(new fst(path::join(info_.get_data_path(),"english_phone_mapping.fst")));
@@ -577,6 +606,13 @@ else
     catch(const io::open_error& e)
       {
       }
+    try
+      {
+        pg2p_fst.reset(new fst(path::join(info_.get_data_path(),"pg2p.fst")));
+      }
+    catch(const io::open_error& e)
+      {
+      }
     try
       {
         pitch_mod_dtree.reset(new dtree(path::join(info_.get_data_path(),"pitch-mod.dt")));
@@ -591,6 +627,13 @@ else
     catch(const io::open_error& e)
       {
       }
+    try
+      {
+        accented_dtree.reset(new dtree(path::join(info_.get_data_path(),"accented.dt")));
+      }
+    catch(const io::open_error& e)
+      {
+      }
     fst msg_fst(path::join(info_.get_data_path(),"msg.fst"));
     std::vector<std::string> src;
     src.push_back("capital");
@@ -606,6 +649,9 @@ else
     whitespace_symbols[32]="sp";
     whitespace_symbols[160]="nbsp";
     register_default_features();
+    auto& labeller=get_hts_labeller();
+    for(const auto& flag: lcfg.ph_flags.get())
+      labeller.define_ph_flag_feature(flag);
   }
 
   void language::register_default_features()
@@ -638,6 +684,7 @@ else
     register_feature(std::shared_ptr<feature_function>(new feat_phrase_numsyls));
     register_feature(std::shared_ptr<feature_function>(new feat_phrase_numwords));
     register_feature(std::shared_ptr<feature_function>(new feat_syl_break));
+    register_feature(std::shared_ptr<feature_function>(new feat_word_break));
     register_feature(std::shared_ptr<feature_function>(new feat_word_stress_pattern));
     register_feature(std::shared_ptr<feature_function>(new feat_phrases_in));
     register_feature(std::shared_ptr<feature_function>(new feat_phrases_out));
@@ -646,15 +693,37 @@ else
     register_feature(std::shared_ptr<feature_function>(new feat_utt_type));
   }
 
-  item& language::append_token(utterance& u,const std::string& text) const
+  item& language::append_subtoken(item& parent_token, const std::string& name, const std::string& pos) const
   {
+    item& token=parent_token.as("TokStructure").append_child();
+    token.set("name",name);
+            token.set("pos",pos);
+            if(((++str::utf8_string_begin(name))==str::utf8_string_end(name))&&(pos=="word"||pos=="lseq"))
+              token.set("one-letter",true);
+            token.set<verbosity_t>("verbosity",(pos=="sym")?verbosity_silent:verbosity_name);
+            return token;
+  }
+
+  item& language::append_token(utterance& u,const std::string& text, bool eos) const
+  {
+    relation& token_rel=u.get_relation("Token",true);
+    relation& tokstruct_rel=u.get_relation("TokStructure",true);
+    item& parent_token=tokstruct_rel.append(token_rel.append());
+    parent_token.set("name",text);
+    if(lcfg.tok_sent)
+      {
+        u.get_relation("TokIn", true).append(parent_token);
+        return parent_token.as("Token");
+      }
     const language_info& lang_info=get_info();
     utf8::uint32_t stress_marker=lang_info.text_settings.stress_marker;
     bool process_stress_marks=lang_info.supports_stress_marks()&&lang_info.text_settings.stress_marker.is_set(true);
     std::vector<utf8::uint32_t> chars;
+    std::vector<std::string> syms;
     std::vector<bool> stress_mask;
     utf8::uint32_t cp;
     std::string::const_iterator it(text.begin());
+    auto pit=it;
     while(it!=text.end())
       {
         cp=utf8::next(it,text.end());
@@ -664,21 +733,28 @@ else
            (lang_info.is_vowel_letter(cp)))
           {
             chars.back()=cp;
+            syms.back()=std::string(pit, it);
             stress_mask.back()=true;
           }
         else
           {
             chars.push_back(cp);
+            syms.push_back(std::string(pit, it));
             stress_mask.push_back(false);
           }
+        pit=it;
+      }
+    if(eos && lcfg.tok_eos)
+      {
+        syms.push_back("eos");
       }
     std::vector<std::string> tokens;
-    if(!tok_fst.translate(chars.begin(),chars.end(),std::back_inserter(tokens)))
-      throw tokenization_error();
-    relation& token_rel=u.get_relation("Token",true);
-    relation& tokstruct_rel=u.get_relation("TokStructure",true);
-    item& parent_token=tokstruct_rel.append(token_rel.append());
-    parent_token.set("name",text);
+    if(!tok_fst.translate(syms.begin(),syms.end(),std::back_inserter(tokens)))
+      throw tokenization_error(text);
+    if(eos && lcfg.tok_eos && !tokens.empty() && tokens.back()=="eos")
+      {
+        tokens.pop_back();
+      }
     std::string name,pos;
     std::vector<utf8::uint32_t>::const_iterator token_start=chars.begin();
     std::vector<utf8::uint32_t>::const_iterator token_end=token_start;
@@ -688,14 +764,9 @@ else
         if(utf8::distance(it->begin(),it->end())>1)
           {
             if(token_start==token_end)
-              throw tokenization_error();
-            item& token=parent_token.append_child();
-            token.set("name",name);
+              throw tokenization_error(text);
             pos=*it;
-            token.set("pos",pos);
-            if(((token_start+1)==token_end)&&(pos=="word"||pos=="lseq"))
-              token.set("one-letter",true);
-            token.set<verbosity_t>("verbosity",(pos=="sym")?verbosity_silent:verbosity_name);
+            item& token=append_subtoken(parent_token, name, pos);
             if((pos=="word")&&(stress.get_state()!=stress_pattern::undefined))
               token.set("stress_pattern",stress);
             stress.reset();
@@ -705,7 +776,7 @@ else
         else
           {
             if(token_end==chars.end())
-              throw tokenization_error();
+              throw tokenization_error(text);
             if(stress_mask[token_end-chars.begin()])
               stress.stress_syllable(1+std::count_if(token_start,token_end,is_vowel_letter(lang_info)));
             utf8::append(*token_end,std::back_inserter(name));
@@ -713,7 +784,7 @@ else
           }
       }
     if(!name.empty())
-      throw tokenization_error();
+      throw tokenization_error(text);
     return parent_token.as("Token");
   }
 
@@ -825,6 +896,7 @@ item& language::append_emoji(utterance& u,const std::string& text) const
 {
   if(!supports_emoji())
     throw language_error("This language doesn't support emoji");
+  on_token_break(u);
     relation& token_rel=u.get_relation("Token",true);
     relation& tokstruct_rel=u.get_relation("TokStructure",true);
     item& token=tokstruct_rel.append(token_rel.append());
@@ -897,9 +969,14 @@ item& language::append_emoji(utterance& u,const std::string& text) const
   {
     if(token.has_children())
       return;
+    const std::string& token_pos=token.get("pos").as<std::string>();
+    if(token_pos=="ph")
+      {
+        token.append_child().set<std::string>("name", "_");
+        return;
+      }
     if(decode_as_english(token))
       return;
-    const std::string& token_pos=token.get("pos").as<std::string>();
     const std::string& token_name=token.get("name").as<std::string>();
     if(token_pos=="word")
       decode_as_word(token,token_name);
@@ -928,6 +1005,7 @@ else
     downcase_fst.translate(str::utf8_string_begin(token_name),str::utf8_string_end(token_name),str::append_string_iterator(word_name));
     item& word=token.append_child();
     word.set("name",word_name);
+    word.set("cname", token_name);
   }
 
   void language::decode_as_word(item& token,const std::string& token_name) const
@@ -1024,6 +1102,84 @@ else
         decode_as_character(token,token_name);
   }
 
+  void language::tokenize(utterance& u) const
+  {
+    if(!lcfg.tok_sent)
+      return;
+    if(!u.has_relation("TokIn"))
+      return;
+    relation& in_rel=u.get_relation("TokIn");
+    const std::string sb{"sb"};
+    const std::string eb{"eb"};
+    const std::string br{"br"};
+    std::vector<std::string> input;
+    for(auto it=in_rel.begin(); it!=in_rel.end(); ++it)
+      {
+        input.push_back(sb);
+        const std::string& name=it->get("name").as<std::string>();
+        str::utf8explode(name, std::back_inserter(input));
+        input.push_back(eb);
+        if(!it->has_next())
+          break;
+        if(it->has_feature("break"))
+          {
+            input.push_back(br);
+            continue;
+          }
+        const std::string& whitespace=it->next().get("whitespace").as<std::string>();
+        str::utf8explode(whitespace, std::back_inserter(input));
+      }
+    std::vector<std::string> output;
+    if(!tok_fst.translate(input.begin(), input.end(), std::back_inserter(output)))
+      throw tokenization_error("");
+    auto in_it=input.cbegin();
+    auto in_tok_it=in_rel.begin();
+    auto out_tok_it=in_tok_it;
+    std::string name;
+    bool in_whitespace=false;
+    bool first_sb=true;
+    for(const auto& out_sym: output)
+      {
+        if(in_it==input.end() || out_sym!=*in_it)
+          {
+            if(name.empty())
+              throw tokenization_error("");
+            if(out_tok_it==in_rel.end())
+              throw tokenization_error(name);
+            append_subtoken(*out_tok_it, name, out_sym);
+            name.clear();
+            out_tok_it=in_tok_it;
+            continue;
+          }
+        ++in_it;
+        if(out_sym==eb)
+          {
+            in_whitespace=true;
+            continue;
+              }
+        if(out_sym==br)
+          continue;
+        if(out_sym==sb)
+          {
+            in_whitespace=false;
+            if(first_sb)
+              first_sb=false;
+            else
+              {
+                ++in_tok_it;
+                if(name.empty())
+                  out_tok_it=in_tok_it;
+              }
+            continue;
+          }
+        if(in_whitespace && name.empty())
+          continue;
+        name.append(out_sym);
+  }
+    if(!name.empty())
+      throw tokenization_error(name);
+  }
+
   void language::do_text_analysis(utterance& u) const
   {
     udict.apply_rules(u);
@@ -1093,6 +1249,15 @@ else
     return ((brk=="NB")?break_none:break_phrase);
   }
 
+void language::on_token_break(utterance& u) const
+{
+  if(!lcfg.tok_sent)
+    return;
+  if(!u.has_relation("TokIn"))
+    return;
+  u.get_relation("TokIn").last().set("break", true);
+}
+
   void language::phrasify(utterance& u) const
   {
     relation& word_rel=u.get_relation("Word");
@@ -1243,6 +1408,20 @@ else
 }
 }
 
+  void language::do_syl_accents(utterance& u) const
+  {
+    if(accented_dtree==nullptr)
+      return;
+    relation& syl_rel=u.get_relation("Syllable");
+    for(relation::iterator syl_iter(syl_rel.begin());syl_iter!=syl_rel.end();++syl_iter)
+      {
+        if(accented_dtree->predict(*syl_iter).as<unsigned int>()!=0)
+          syl_iter->set<std::string>("accented","1");
+        else
+          syl_iter->set<std::string>("accented","0");
+      }
+  }
+
   std::vector<std::string> language::get_english_word_transcription(const item& word) const
   {
     const language_list& languages=get_info().get_all_languages();
@@ -1275,6 +1454,57 @@ else
 }
   }
 
+void language::post_g2p(utterance& u) const
+{
+  if(pg2p_fst.get()==0)
+    return;
+  relation& trans_rel=u.get_relation("Transcription");
+  if(trans_rel.empty())
+    return;
+  std::vector<std::string> in_syms;
+    for(relation::const_iterator word_iter=trans_rel.begin();word_iter!=trans_rel.end();++word_iter)
+      {
+        const item& word=*word_iter;
+        if(word.has_prev())
+          in_syms.push_back(word.as("Phrase").has_prev()?"#":"-");
+        std::transform(word.begin(),word.end(),std::back_inserter(in_syms), [](const item& i) {return i.get_name();});
+      }
+    std::vector<std::string> out_syms;
+if(!pg2p_fst->translate(in_syms.begin(), in_syms.end(), std::back_inserter(out_syms)))
+  throw post_g2p_error();
+ auto out_iter=out_syms.begin();
+ for(relation::iterator word_iter=trans_rel.begin();word_iter!=trans_rel.end();++word_iter)
+      {
+        item& word=*word_iter;
+        if(word.has_prev())
+          {
+            if(out_iter==out_syms.end())
+              throw post_g2p_error(word);
+            ++out_iter;
+          }
+        word.clear();
+        for(;out_iter!=out_syms.end() && *out_iter!="#" && *out_iter!="-"; ++out_iter)
+          {
+            if(str::startswith(*out_iter, "_"))
+              {
+                const auto flag=out_iter->substr(1);
+                if(!lcfg.ph_flags.includes(flag) || !word.has_children())
+                  throw post_g2p_error(word);
+                word.last_child().set("ph_flag_"+flag, std::string("1"));
+                continue;
+              }
+            word.append_child().set("name", *out_iter);
+          }
+      }
+}
+
+  void language::set_user_phones(item& word) const
+  {
+    const std::string s=word.as("TokStructure").parent().get("name").as<std::string>();
+    str::tokenizer<str::is_space> t(s);
+    std::copy(t.begin(), t.end(), word.back_inserter());
+  }
+
   void language::do_g2p(utterance& u) const
   {
     relation& word_rel=u.get_relation("Word");
@@ -1282,11 +1512,21 @@ else
     relation& trans_rel=u.add_relation("Transcription");
     for(relation::iterator word_iter=word_rel.begin();word_iter!=word_rel.end();++word_iter)
       {
-        before_g2p(*word_iter);
+        const bool ph=(word_iter->as("TokStructure").parent().get("pos").as<std::string>()=="ph");
+        if(!ph)
+          before_g2p(*word_iter);
         item& word=trans_rel.append(*word_iter);
-        assign_pronunciation(word);
+        if(ph)
+          set_user_phones(word);
+        else
+          assign_pronunciation(word);
         if(!word.has_children())
           throw g2p_error(word);
+      }
+    post_g2p(u);
+    for(relation::iterator word_iter=trans_rel.begin();word_iter!=trans_rel.end();++word_iter)
+      {
+        item& word=*word_iter;
         std::copy(word.begin(),word.end(),seg_rel.back_inserter());
       }
   }
@@ -1299,32 +1539,50 @@ else
     std::vector<std::string> result;
     item::iterator seg_start,seg_end,seg_iter;
     std::string seg_name;
-    std::size_t n;
+    std::size_t i, n;
+    std::string feat_name;
     for(relation::iterator word_iter=trans_rel.begin();word_iter!=trans_rel.end();++word_iter)
       {
         item& word_with_syls=sylstruct_rel.append(*word_iter);
         seg_start=word_iter->begin();
         seg_end=word_iter->end();
         if(!syl_fst.translate(seg_start,seg_end,std::back_inserter(result)))
-          throw syllabification_error();
+          throw syllabification_error(*word_iter);
         word_with_syls.append_child().set<std::string>("stress","0");
+        word_with_syls.last_child().set<std::string>("accented","0");
         seg_iter=seg_start;
         for(std::vector<std::string>::const_iterator pos=result.begin();pos!=result.end();++pos)
           {
+            if((*pos)[0]=='_')
+              {
+                feat_name=pos->substr(1);
+                word_with_syls.last_child().set<std::string>(feat_name, "1");
+                if(feat_name=="accented")
+                  word_with_syls.last_child().set<std::string>("stress","1");
+                continue;
+              }
             if(seg_iter==seg_end)
-              throw syllabification_error();
+              throw syllabification_error(*word_iter);
             if(*pos==".")
-              word_with_syls.append_child().set<std::string>("stress","0");
+              {
+                word_with_syls.append_child().set<std::string>("stress","0");
+                word_with_syls.last_child().set<std::string>("accented","0");
+              }
             else
               {
                 seg_name=seg_iter->get("name").as<std::string>();
+                i=0;
+                if(seg_name[i]=='/')
+                  ++i;
                 n=seg_name.size()-1;
                 if((seg_name[n]=='0')||(seg_name[n]=='1'))
                   {
                     if(seg_name[n]=='1')
                       word_with_syls.last_child().set<std::string>("stress","1");
-                    seg_iter->set("name",seg_name.substr(0,n));
+                    seg_iter->set("name",seg_name.substr(i,n-i));
                   }
+                else if(i>0)
+                  seg_iter->set("name",seg_name.substr(i));
                 word_with_syls.last_child().append_child(*seg_iter);
                 ++seg_iter;
               }
@@ -1441,11 +1699,14 @@ else
     register_language<kyrgyz_info>("Kyrgyz",1);
     register_language<tatar_info>("Tatar",1);
     register_language<brazilian_portuguese_info>("Brazilian-Portuguese",1);
+    register_language<macedonian_info>("Macedonian",1);
     for(std::vector<std::string>::const_iterator it1=language_paths.begin();it1!=language_paths.end();++it1)
       {
         logger.log(tag,RHVoice_log_level_info,std::string("Path: ")+(*it1));
         resource_description desc("language",*it1);
         logger.log(tag,RHVoice_log_level_info,std::string("Language resource: ")+desc.name.get()+std::string(", format: ")+str::to_string(desc.format.get())+std::string(", revision: ")+str::to_string(desc.revision.get()));
+        if(desc.data_only && !desc.name.get().empty())
+          register_language<data_only_language_info>(desc.name,1);
         Creators::const_iterator it2=creators.find(language_id(desc.name,desc.format));
         if(it2==creators.end())
           {
