@@ -11,7 +11,7 @@
 /* GNU Lesser General Public License for more details. */
 
 /* You should have received a copy of the GNU Lesser General Public License */
-/* along with this program.  If not, see <http://www.gnu.org/licenses/>. */
+/* along with this program.  If not, see <https://www.gnu.org/licenses/>. */
 
 #include <utility>
 #include "core/utterance.hpp"
@@ -32,7 +32,7 @@ namespace RHVoice
   {
     relation_map::iterator it(relations.find(name));
     if(it==relations.end())
-      throw relation_not_found();
+      throw relation_not_found(name);
     relations.erase(it);
   }
 }
