@@ -1,4 +1,5 @@
-/* Copyright (C) 2012, 2019, 2020  Olga Yakovleva <yakovleva.o.v@gmail.com> */
+/* Copyright (C) 2012, 2019, 2020, 2021  Olga Yakovleva <olga@rhvoice.org> */
+/* Copyright (C) 2022 Non-Routine LLC.  <lp@louderpages.org> */
 
 /* This program is free software: you can redistribute it and/or modify */
 /* it under the terms of the GNU Lesser General Public License as published by */
@@ -11,7 +12,7 @@
 /* GNU Lesser General Public License for more details. */
 
 /* You should have received a copy of the GNU Lesser General Public License */
-/* along with this program.  If not, see <http://www.gnu.org/licenses/>. */
+/* along with this program.  If not, see <https://www.gnu.org/licenses/>. */
 
 #include <memory>
 #include <string>
@@ -71,7 +72,8 @@ namespace RHVoice
         command_token,
         command_mark,
         command_audio,
-        command_break
+        command_break,
+        command_phones
       };
 
     class abstract_command: public utterance_function
@@ -88,6 +90,8 @@ namespace RHVoice
       {
         return true;
       }
+
+      virtual void set_eos() {}
     };
 
     typedef std::shared_ptr<abstract_command> command_ptr;
@@ -114,11 +118,14 @@ namespace RHVoice
         std::copy(prev_token.whitespace.begin(),prev_token.whitespace.end(),str::utf8_inserter(std::back_inserter(whitespace)));
     }
 
+      void set_eos() {eos=true;}
+
       void execute(utterance& u) const;
 
     protected:
       std::string name,whitespace;
       std::size_t position,length;
+      bool eos{false};
     };
 
     class append_chars: public append_token
@@ -199,6 +206,7 @@ namespace RHVoice
       void execute(utterance& u) const
       {
         u.get_relation("Event",true).append().set("audio",src);
+        u.get_language().on_token_break(u);
       }
 
       bool has_text() const
@@ -226,13 +234,29 @@ namespace RHVoice
       void execute(utterance& u) const
       {
         if(u.has_relation("TokStructure"))
-          u.get_relation("TokStructure").last().set("break_strength",strength);
+          {
+            u.get_relation("TokStructure").last().set("break_strength",strength);
+            if(strength>=break_phrase)
+              u.get_language().on_token_break(u);
+          }
       }
 
     private:
       break_strength strength;
     };
 
+    class append_phones: public append_token
+    {
+    public:
+      explicit append_phones(const text_token& prev_token,const text_token& token):
+        append_token(prev_token,token)
+
+    {
+    }
+
+      void execute(utterance& u) const;
+    };
+
     std::list<command_ptr> commands;
     text_token prev_token,next_token;
     document* parent;
@@ -488,7 +512,7 @@ namespace RHVoice
     if(text_start==text_end)
       return text_start;
     text_iterator token_end=text_start;
-    if(markup_info.say_as==content_key)
+    if(markup_info.say_as==content_key || markup_info.say_as==content_phones)
       token_end=text_end;
     else if((markup_info.say_as!=content_text)&&str::isspace(*text_start))
       ++token_end;
@@ -581,6 +605,8 @@ namespace RHVoice
           commands.push_back(command_ptr(new append_token(prev_token,next_token)));
         else if(next_token.type==content_emoji)
           commands.push_back(command_ptr(new append_emoji(prev_token,next_token)));
+        else if(next_token.type==content_phones)
+          commands.push_back(command_ptr(new append_phones(prev_token,next_token)));
         else if(next_token.type==content_key)
           commands.push_back(command_ptr(new append_key(prev_token,next_token)));
         else
@@ -633,6 +659,8 @@ namespace RHVoice
     parser.add_element_handler(audio_handler);
     ssml::break_handler<char_type> break_handler;
     parser.add_element_handler(break_handler);
+    ssml::phoneme_handler<char_type> phoneme_handler;
+    parser.add_element_handler(phoneme_handler);
     parser.parse(text_start,text_end,*doc_ptr);
     return doc_ptr;
   }
