@@ -11,12 +11,16 @@
 /* GNU Lesser General Public License for more details. */
 
 /* You should have received a copy of the GNU Lesser General Public License */
-/* along with this program.  If not, see <http://www.gnu.org/licenses/>. */
+/* along with this program.  If not, see <https://www.gnu.org/licenses/>. */
 
 #ifndef RHVOICE_HTS_ENGINE_SETTING_HPP
 #define RHVOICE_HTS_ENGINE_SETTING_HPP
 
 #include "property.hpp"
+#ifndef ENABLE_MAGE
+#include "config.h"
+#endif
+
 
 namespace RHVoice
 {
@@ -27,7 +31,7 @@ namespace RHVoice
       enum_string_property("hts_engine")
     {
       define("standard");
-#ifdef ENABLE_MAGE
+#if ENABLE_MAGE
       define("mage");
       set_default_value("mage");
 #else
