@@ -11,7 +11,7 @@
 /* GNU Lesser General Public License for more details. */
 
 /* You should have received a copy of the GNU Lesser General Public License */
-/* along with this program.  If not, see <http://www.gnu.org/licenses/>. */
+/* along with this program.  If not, see <https://www.gnu.org/licenses/>. */
 
 #ifndef RHVOICE_STD_HTS_ENGINE_IMPL_HPP
 #define RHVOICE_STD_HTS_ENGINE_IMPL_HPP
@@ -44,8 +44,10 @@ namespace RHVoice
     void set_time_info();
     void set_speed();
     void edit_pitch();
+    void output_debug_info();
 
     std::unique_ptr<_HTS_Engine> engine;
+    std::size_t base_frame_shift;
   };
 }
 #endif
