@@ -53,6 +53,11 @@ if ( @ARGV < 1 ) {
 # load configuration variables
 require( $ARGV[0] );
 
+use Parallel::ForkManager;
+$pfm = Parallel::ForkManager->new($NUMPROC);
+$pfm->set_waitpid_blocking_sleep(0);  # true blocking calls enabled
+$pfm->run_on_finish(\&merge_clustered_model);
+
 # model structure
 foreach $set (@SET) {
    $vSize{$set}{'total'}   = 0;
@@ -115,6 +120,7 @@ foreach $set (@SET) {
    $monommf{$set} = "$model{$set}/monophone.mmf";
    $fullmmf{$set} = "$model{$set}/fullcontext.mmf";
    $clusmmf{$set} = "$model{$set}/clustered.mmf";
+   $jmmf{$set} = $clusmmf{$set};
    $untymmf{$set} = "$model{$set}/untied.mmf";
    $reclmmf{$set} = "$model{$set}/re_clustered.mmf";
    $rclammf{$set} = "$model{$set}/re_clustered_all.mmf";
@@ -140,6 +146,7 @@ foreach $set (@SET) {
    foreach $type ( @{ $ref{$set} } ) {
       $cnv{$type} = "$hed{$set}/cnv_$type.hed";
       $cxc{$type} = "$hed{$set}/cxc_$type.hed";
+      $jm{$type} = "$hed{$set}/jm_$type.hed";
    }
 }
 
@@ -160,6 +167,17 @@ foreach $set (@SET) {
    }
 }
 
+# Reference trees
+foreach $set (@SET) {
+   foreach $type ( @{ $ref{$set} } ) {
+       $reftree{$type} = "$datdir/reftrees/${type}.inf";
+       $reftreeflag{$type} = -f $reftree{$type} ;
+      $refLT{$type} = $reftreeflag{$type} ? "LT \"$reftree{$type}\"\n" : "";
+      $HHEdq{$type} = $reftreeflag{$type} ? "-q 1" : "";
+   }
+}
+
+
 # converted model & tree files for hts_engine
 $voice = "$prjdir/voices/qst${qnum}/ver${ver}";
 foreach $set (@SET) {
@@ -475,7 +493,9 @@ if ($ERST1) {
    }
 }
 
-# HHEd (tree-based context clustering)
+for($cli=1 ; $cli < $NUMCL ; $cli++) {
+    # HHEd (tree-based context clustering)
+    print("\n\nIteration $cli of tree-based context clustering\n");
 if ($CXCL1) {
    print_time("tree-based context clustering");
 
@@ -484,18 +504,22 @@ if ($CXCL1) {
 
    # tree-based clustering
    foreach $set (@SET) {
-      shell("cp $fullmmf{$set} $clusmmf{$set}");
+       if($cli == 1) {
+           shell("cp $fullmmf{$set} $clusmmf{$set}");
+       } else {
+           shell("cp $untymmf{$set} $clusmmf{$set}");
+       }
 
-      $footer = "";
       foreach $type ( @{ $ref{$set} } ) {
          if ( $strw{$type} > 0.0 ) {
-            make_edfile_state($type);
-            shell("$HHEd{'trn'} -C $cfg{$type} -H $clusmmf{$set} $mdl{$type} -w $clusmmf{$set} $cxc{$type} $lst{'ful'}");
-            $footer .= "_$type";
-            shell("gzip -c $clusmmf{$set} > $clusmmf{$set}$footer.gz");
+             make_edfile_state($type);
+             $pfm->start($type) and next;
+            shell("$HHEd{'trn'} $HHEdq{$type} -C $cfg{$type} -H $clusmmf{$set} $mdl{$type} -w $clusmmf{$set}_$type $cxc{$type} $lst{'ful'}");
+             $pfm->finish;
          }
       }
    }
+   $pfm->wait_all_children;
 }
 
 # HERest (embedded reestimation (clustered))
@@ -524,12 +548,18 @@ if ($UNTIE) {
 }
 
 # fix variables
-foreach $set (@SET) {
-   $stats{$set} .= ".untied";
+    foreach $set (@SET) {
+        if($cli == 1) {
+            $stats{$set} .= ".untied";
    foreach $type ( @{ $ref{$set} } ) {
       $tre{$type} .= ".untied";
       $cxc{$type} .= ".untied";
    }
+        }
+
+    if($cli == ($NUMCL - 1)) {
+        $jmmf{$set} = $reclmmf{$set};
+    }
 }
 
 # HERest (embedded reestimation (untied))
@@ -541,6 +571,7 @@ if ($ERST3) {
    print("\n\nEmbedded Re-estimation for untied mmfs\n");
    shell("$HERest{'ful'} -H $untymmf{'cmp'} -N $untymmf{'dur'} -M $model{'cmp'} -R $model{'dur'} $opt $lst{'ful'} $lst{'ful'}");
 }
+}
 
 # HHEd (tree-based context clustering)
 if ($CXCL2) {
@@ -552,17 +583,14 @@ if ($CXCL2) {
    # tree-based clustering
    foreach $set (@SET) {
       shell("cp $untymmf{$set} $reclmmf{$set}");
-
-      $footer = "";
       foreach $type ( @{ $ref{$set} } ) {
-         make_edfile_state($type);
-         shell("$HHEd{'trn'} -C $cfg{$type} -H $reclmmf{$set} $mdl{$type} -w $reclmmf{$set} $cxc{$type} $lst{'ful'}");
-
-         $footer .= "_$type";
-         shell("gzip -c $reclmmf{$set} > $reclmmf{$set}$footer.gz");
+          make_edfile_state($type);
+          $pfm->start($type) and next;
+          shell("$HHEd{'trn'} $HHEdq{$type} -C $cfg{$type} -H $reclmmf{$set} $mdl{$type} -w $reclmmf{$set}_$type $cxc{$type} $lst{'ful'}");
+          $pfm->finish;
       }
-      shell("gzip -c $reclmmf{$set} > $reclmmf{$set}.nonembedded.gz");
    }
+   $pfm->wait_all_children;
 }
 
 # HERest (embedded reestimation (re-clustered))
@@ -1585,6 +1613,8 @@ sub make_edfile_state($) {
    print EDFILE "// questions for decision tree-based context clustering\n";
    print EDFILE @lines;
    print EDFILE "TR 3\n\n";
+   print EDFILE "// load reference trees\n";
+   print EDFILE "$refLT{$type}\n";
    print EDFILE "// construct decision trees\n";
 
    for ( $i = 2 ; $i <= $nstate{ $t2s{$type} } + 1 ; $i++ ) {
@@ -2561,4 +2591,24 @@ sub make_mspf($) {
    }
 }
 
+sub merge_clustered_model {
+    my ($pid, $exit, $type) = @_;
+    my ($set) = $t2s{$type};
+   if ( $exit / 256 != 0 ) {
+      die "Error while clustering $type\n";
+   }
+    if ($type eq "dur") {
+        print "Copying clustered duration model\n";
+        shell("cp $jmmf{$set}_${type} $jmmf{$set}");
+        return;
+   }
+    print "Merging clustered $type\n";
+    open( EDFILE, ">$jm{$type}" ) || die "Cannot open $!";
+    for ( $i = 2 ; $i <= $nState + 1 ; $i++ ) {
+        print EDFILE "JM $jmmf{$set}_${type} {*.state[${i}].stream[$strb{$type}-$stre{$type}]}\n";
+    }
+    close(EDFILE);
+    shell("$HHEd{'trn'} -H $jmmf{$set} -w $jmmf{$set} $jm{$type} $lst{'ful'}");
+}
+
 ##################################################################################################
